<?php 

/* -----------------------------------------------------------------------------
  class name: manage_tables
  class version: 1.2
  date: 2007-12-02
  ------------------------------------------------------------------------------
  author: grum at grum.dnsalias.com
  << May the Little SpaceFrog be with you >>
  ------------------------------------------------------------------------------
  
  this class provides base functions to manage tables while plugin installation
    - constructor manage_tables($tables)
    - (public) function create_tables($tables_def)
    - (public) function update_tables_fields($tables_alteration)
    - (public) function drop_tables()
    - (public) function rename_tables($list)  -v1.1
    - (public) function tables_exists()  -v1.1
    - (public) function export($filename, $options, $tables)  -v1.2

  ------------------------------------------------------------------------------
  v1.1 + add rename_tables($list) function
       + add tables_exists() function
  v1.2 + add export($filename, $options, $tables) function

   -------------------------------------------------------------------------- */
class manage_tables
{
  var $tables;    //array of tables names

  function manage_tables($tables)
  {
    $this->tables = $tables;
  }
  
  /* 
      create tables
      $tables_def is an array of SQL CREATE queries

      return true if everything is ok, otherwise tablename 
  */
  function create_tables($tables_def)
  {
    //deleting tables if exists
    $this->drop_tables();

    for($i=0;$i<count($tables_def);$i++)
    {
      $result=pwg_query($tables_def[$i]);
      if(!$result)
      {
        //if an error occurs, deleting created tables
        $this->drop_tables();
        return(false);
      }
    }
    return(true);
  }

  /* update tables definition 
     $tables_alteration : array of arrays
      example :
      $tables_alteration['table1']=array(
        "attribute1" => " ADD COLUMN `attribute1` text null default ''",
        "attribute2" => " ADD COLUMN `attribute2` text null default ''"));
      $tables_alteration['table2']=array(
        "attribute1" => " ADD COLUMN `attribute1` text null default ''",
        "attribute2" => " ADD COLUMN `attribute2` text null default ''"));

      return true if no error, otherwise return table.fields of error 
  */
  function update_tables_fields($tables_alteration)
  {
    if(!is_array($tables_alteration))
    {
      return('');
    }

    reset($tables_alteration);
    while (list($key, $val) = each($tables_alteration))
    {
      $sql="SHOW COLUMNS FROM $key";
      $result=pwg_query($sql);
      if($result)
      {
        $columns=array();
        while($row=mysql_fetch_assoc($result))
        { 
          array_push($columns, $row['Field']); 
        }

        reset($val);
        while (list($attname, $sql) = each($val))
        {
          if(!in_array($attname, $columns))
          {
            $result=pwg_query("ALTER TABLE `$key` ".$sql);
            if(!$result)
            {
              return($key.".".$attname);
            }
          }
        }
      }
    }
    return(true);
  }


  /* 
      delete tables listed in $this->tables_list
  */
  function drop_tables()
  {
    foreach($this->tables as $key => $table_name)
    {
      $sql="DROP TABLE IF EXISTS ".$table_name;
      $result=pwg_query($sql);
    }
  }

  /*
      rename tables name of list
        $list is an array('old_name' => 'new_name')
      return true if ok, else old table name
  */
  function rename_tables($list)
  {
    $tmplist=array_flip($this->tables);
    foreach($list as $key => $val)
    {
      if(isset($tmplist[$key]))
      {
        $this->tables[$tmplist[$key]] = $val;
        $sql="ALTER TABLE `$key` RENAME TO `$val`";
        if(!pwg_query($sql))
        {
          return($key);
        }
      }
      else
      {
        return($key);
      }
    }
    return(true);
  }

  /*
    return true if all listed tables exists
  */
  function tables_exists()
  {
    $list=array_flip($this->tables);
    $sql="SHOW TABLES";
    $result=pwg_query($sql);
    if($result)
    {
      while($row=mysql_fetch_row($result))
      {
        if(isset($list[$row[0]]))
        {
          array_splice($list, $row[0],1);
        }
      }
    }
    if(count($list)>0)
    {
      return(false);
    }
    else
    {
      return(true);
    }
  }

  /*
    export all tables as SQL in a text file
      $filename : name of the file
      $options : array of options like
                    array(
                      'drop' => true/false,  //add DROP TABLE statements
                      'create' => true/false,  //add CREATE TABLE statements
                      'insert' => true/false,  //add INSERT statements
                    )
      $tables : array of tables names to export
                    array('tablename1', 'tablenamen', 'tablename3', ...)
                  if empty, assume that all tables have to be exported
                  
  */
  function export($filename, $options=array(), $tables=array())
  {
    $defaultopt=array(
      'drop' => true,
      'create' => true,
      'insert' => true
    );

    if(!isset($options['drop']))
    {
      $options['drop']=$defaultopt['drop'];
    }
    if(!isset($options['create']))
    {
      $options['create']=$defaultopt['create'];
    }
    if(!isset($options['insert']))
    {
      $options['insert']=$defaultopt['insert'];
    }
    if(count($tables)==0)
    {
      $tables=$this->tables;
    }

    $returned=true;
    $text='';
    foreach($tables as $key => $val)
    {
      $text.="

-- ***************************************************************
-- * Statements for ".$this->tables[$key]." table
-- ***************************************************************
";

      if($options['drop'])
      {
        $text.=sprintf("DROP TABLE `%s`;\n", $this->tables[$key]);
      }

      if($options['create'])
      {
        $sql='SHOW CREATE TABLE '.$this->tables[$key];
        $result=pwg_query($sql);
        if($result)
        {
          while($row=mysql_fetch_row($result))
          {
            $text.=sprintf("%s;\n", $row[1]);
          }
        }
        else
        {
          $returned=false;
        }
      }

      if($options['insert'])
      {
        $sql='SELECT * FROM '.$this->tables[$key];
        $result=pwg_query($sql);
        if($result)
        {
          while($row=mysql_fetch_row($result))
          {
            foreach($row as $key2 => $val2)
            {
              $row[$key2]="'".addslashes($val2)."'";
            }
            $text.=sprintf("INSERT INTO `%s` VALUES(%s);\n", $this->tables[$key], implode(', ', $row));
          }
        }
        else
        {
          $returned=false;
        }
      }
    }
    $fhandle=fopen($filename, 'wb');
    if($fhandle)
    {
      fwrite($fhandle, $text);
      fclose($fhandle);
    }
    else
    {
      $returned=false;
    }
    return($returned);
  }

} //class


?>